@@ -1,5 +1,6 @@
 #!/bin/bash
 
+home_dir=$(pwd)
 ccache -s
 
 MISSES=$(ccache -s | grep "cache miss" | grep -Po "\d+")
@@ -11,15 +12,16 @@ if [ x"$MISSES" == x"0" ]; then
 fi
 
 if [ "$IRON_TOKEN" ]; then
-    rm -rf ~/ccache.7z
+
+    rm -rf $HOME/ccache.7z
 
     tar cf - $HOME/.ccache \
     "$TRAVIS_BUILD_DIR"/pandas/{index,algos,lib,tslib,parser,hashtable}.c \
     "$TRAVIS_BUILD_DIR"/pandas/src/{sparse,testing}.c \
     "$TRAVIS_BUILD_DIR"/pandas/msgpack.cpp  \
-    |  7za a -si ~/ccache.7z
+    |  7za a -si $HOME/ccache.7z
 
-    split -b 500000 -d ~/ccache.7z ~/ccache.
+    split -b 500000 -d $HOME/ccache.7z $HOME/ccache.
 
     python ci/ironcache/put.py
 fi;
