@@ -1,26 +1,31 @@
-from pandas import compat
 import sys
 import itertools
 import functools
 
 import numpy as np
 
-from pandas.core.common import isnull, notnull, _values_from_object, is_float
-import pandas.core.common as com
-import pandas.lib as lib
-import pandas.algos as algos
-import pandas.hashtable as _hash
-import pandas.tslib as tslib
-
-from pandas.compat import builtins
-
-
 try:
     import bottleneck as bn
     _USE_BOTTLENECK = True
 except ImportError:  # pragma: no cover
     _USE_BOTTLENECK = False
 
+import pandas.core.common as com
+import pandas.hashtable as _hash
+from pandas import compat, lib, algos, tslib
+from pandas.compat import builtins
+from pandas.core.common import (isnull, notnull, _values_from_object,
+                                _maybe_upcast_putmask,
+                                ensure_float, _ensure_float64,
+                                _ensure_int64, _ensure_object,
+                                is_float, is_integer, is_complex,
+                                is_float_dtype, _is_floating_dtype,
+                                is_complex_dtype, is_integer_dtype,
+                                is_bool_dtype, is_object_dtype,
+                                is_datetime64_dtype, is_timedelta64_dtype,
+                                _is_datetime_or_timedelta_dtype,
+                                _is_int_or_datetime_dtype, _is_any_int_dtype)
+
 
 class disallow(object):
 
@@ -75,7 +80,8 @@ class bottleneck_switch(object):
                         result.fill(0)
                         return result
 
-                if _USE_BOTTLENECK and skipna and _bn_ok_dtype(values.dtype, bn_name):
+                if _USE_BOTTLENECK and skipna and _bn_ok_dtype(values.dtype,
+                                                               bn_name):
                     result = bn_func(values, axis=axis, **kwds)
 
                     # prefer to treat inf/-inf as NA, but must compute the func
@@ -94,7 +100,8 @@ class bottleneck_switch(object):
 
 def _bn_ok_dtype(dt, name):
     # Bottleneck chokes on datetime64
-    if dt != np.object_ and not issubclass(dt.type, (np.datetime64, np.timedelta64)):
+    if (not is_object_dtype(dt) and
+            not _is_datetime_or_timedelta_dtype(dt)):
 
         # bottleneck does not properly upcast during the sum
         # so can overflow
@@ -105,14 +112,18 @@ def _bn_ok_dtype(dt, name):
         return True
     return False
 
+
 def _has_infs(result):
     if isinstance(result, np.ndarray):
         if result.dtype == 'f8':
-            return lib.has_infs_f8(result)
+            return lib.has_infs_f8(result.ravel())
         elif result.dtype == 'f4':
-            return lib.has_infs_f4(result)
+            return lib.has_infs_f4(result.ravel())
+    try:
+        return np.isinf(result).any()
+    except (TypeError, NotImplementedError) as e:
+        # if it doesn't support infs, then it can't have infs
         return False
-    return np.isinf(result) or np.isneginf(result)
 
 
 def _get_fill_value(dtype, fill_value=None, fill_value_typ=None):
@@ -165,8 +176,7 @@ def _get_values(values, skipna, fill_value=None, fill_value_typ=None,
 
         # promote if needed
         else:
-            values, changed = com._maybe_upcast_putmask(values, mask,
-                                                        fill_value)
+            values, changed = _maybe_upcast_putmask(values, mask, fill_value)
 
     elif copy:
         values = values.copy()
@@ -175,33 +185,29 @@ def _get_values(values, skipna, fill_value=None, fill_value_typ=None,
 
     # return a platform independent precision dtype
     dtype_max = dtype
-    if dtype.kind == 'i' and not issubclass(
-        dtype.type, (np.bool, np.datetime64, np.timedelta64)):
+    if is_integer_dtype(dtype) or is_bool_dtype(dtype):
         dtype_max = np.int64
-    elif dtype.kind in ['b'] or issubclass(dtype.type, np.bool):
-        dtype_max = np.int64
-    elif dtype.kind in ['f']:
+    elif is_float_dtype(dtype):
         dtype_max = np.float64
 
     return values, mask, dtype, dtype_max
 
 
 def _isfinite(values):
-    if issubclass(values.dtype.type, (np.timedelta64, np.datetime64)):
+    if _is_datetime_or_timedelta_dtype(values):
         return isnull(values)
-    elif isinstance(values.dtype, object):
-        return ~np.isfinite(values.astype('float64'))
-
-    return ~np.isfinite(values)
+    if (is_complex_dtype(values) or is_float_dtype(values) or
+            is_integer_dtype(values) or is_bool_dtype(values)):
+        return ~np.isfinite(values)
+    return ~np.isfinite(values.astype('float64'))
 
 
 def _na_ok_dtype(dtype):
-    return not issubclass(dtype.type, (np.integer, np.datetime64,
-                                       np.timedelta64))
+    return not _is_int_or_datetime_dtype(dtype)
 
 
 def _view_if_needed(values):
-    if issubclass(values.dtype.type, (np.datetime64, np.timedelta64)):
+    if _is_datetime_or_timedelta_dtype(values):
         return values.view(np.int64)
     return values
 
@@ -209,12 +215,12 @@ def _view_if_needed(values):
 def _wrap_results(result, dtype):
     """ wrap our results if needed """
 
-    if issubclass(dtype.type, np.datetime64):
+    if is_datetime64_dtype(dtype):
         if not isinstance(result, np.ndarray):
             result = lib.Timestamp(result)
         else:
             result = result.view(dtype)
-    elif issubclass(dtype.type, np.timedelta64):
+    elif is_timedelta64_dtype(dtype):
         if not isinstance(result, np.ndarray):
 
             # this is a scalar timedelta result!
@@ -247,7 +253,7 @@ def nanall(values, axis=None, skipna=True):
 @bottleneck_switch(zero_value=0)
 def nansum(values, axis=None, skipna=True):
     values, mask, dtype, dtype_max = _get_values(values, skipna, 0)
-    the_sum = values.sum(axis,dtype=dtype_max)
+    the_sum = values.sum(axis, dtype=dtype_max)
     the_sum = _maybe_null_out(the_sum, axis, mask)
 
     return _wrap_results(the_sum, dtype)
@@ -260,7 +266,7 @@ def nanmean(values, axis=None, skipna=True):
     the_sum = _ensure_numeric(values.sum(axis, dtype=dtype_max))
     count = _get_counts(mask, axis)
 
-    if axis is not None:
+    if axis is not None and getattr(the_sum, 'ndim', False):
         the_mean = the_sum / count
         ct_mask = count == 0
         if ct_mask.any():
@@ -286,6 +292,9 @@ def nanmedian(values, axis=None, skipna=True):
     if values.dtype != np.float64:
         values = values.astype('f8')
 
+    if axis is None:
+        values = values.ravel()
+
     notempty = values.size
 
     # an array from a frame
@@ -308,23 +317,10 @@ def nanmedian(values, axis=None, skipna=True):
     return _wrap_results(get_median(values), dtype) if notempty else np.nan
 
 
-@disallow('M8')
-@bottleneck_switch(ddof=1)
-def nanvar(values, axis=None, skipna=True, ddof=1):
-    if not isinstance(values.dtype.type, np.floating):
-        values = values.astype('f8')
-
-    mask = isnull(values)
-
-    if axis is not None:
-        count = (values.shape[axis] - mask.sum(axis)).astype(float)
-    else:
-        count = float(values.size - mask.sum())
+def _get_counts_nanvar(mask, axis, ddof):
+    count = _get_counts(mask, axis)
 
     d = count-ddof
-    if skipna:
-        values = values.copy()
-        np.putmask(values, mask, 0)
 
     # always return NaN, never inf
     if np.isscalar(count):
@@ -332,21 +328,50 @@ def nanvar(values, axis=None, skipna=True, ddof=1):
             count = np.nan
             d = np.nan
     else:
-        mask = count <= ddof
-        if mask.any():
-            np.putmask(d, mask, np.nan)
-            np.putmask(count, mask, np.nan)
+        mask2 = count <= ddof
+        if mask2.any():
+            np.putmask(d, mask2, np.nan)
+            np.putmask(count, mask2, np.nan)
+    return count, d
+
+
+@disallow('M8')
+@bottleneck_switch(ddof=1)
+def nanvar(values, axis=None, skipna=True, ddof=1):
+    if not _is_floating_dtype(values):
+        values = values.astype('f8')
+
+    mask = isnull(values)
+
+    count, d = _get_counts_nanvar(mask, axis, ddof)
+
+    if skipna:
+        values = values.copy()
+        np.putmask(values, mask, 0)
 
     X = _ensure_numeric(values.sum(axis))
     XX = _ensure_numeric((values ** 2).sum(axis))
     return np.fabs((XX - X ** 2 / count) / d)
 
+
+def nansem(values, axis=None, skipna=True, ddof=1):
+    var = nanvar(values, axis, skipna, ddof=ddof)
+
+    if not _is_floating_dtype(values):
+        values = values.astype('f8')
+    mask = isnull(values)
+    count, _ = _get_counts_nanvar(mask, axis, ddof)
+
+    return np.sqrt(var)/np.sqrt(count)
+
+
 @bottleneck_switch()
 def nanmin(values, axis=None, skipna=True):
-    values, mask, dtype, dtype_max = _get_values(values, skipna, fill_value_typ='+inf')
+    values, mask, dtype, dtype_max = _get_values(values, skipna,
+                                                 fill_value_typ='+inf')
 
     # numpy 1.6.1 workaround in Python 3.x
-    if (values.dtype == np.object_ and compat.PY3):
+    if is_object_dtype(values) and compat.PY3:
         if values.ndim > 1:
             apply_ax = axis if axis is not None else 0
             result = np.apply_along_axis(builtins.min, apply_ax, values)
@@ -359,7 +384,7 @@ def nanmin(values, axis=None, skipna=True):
         if ((axis is not None and values.shape[axis] == 0)
                 or values.size == 0):
             try:
-                result = com.ensure_float(values.sum(axis,dtype=dtype_max))
+                result = ensure_float(values.sum(axis, dtype=dtype_max))
                 result.fill(np.nan)
             except:
                 result = np.nan
@@ -372,10 +397,11 @@ def nanmin(values, axis=None, skipna=True):
 
 @bottleneck_switch()
 def nanmax(values, axis=None, skipna=True):
-    values, mask, dtype, dtype_max = _get_values(values, skipna, fill_value_typ='-inf')
+    values, mask, dtype, dtype_max = _get_values(values, skipna,
+                                                 fill_value_typ='-inf')
 
     # numpy 1.6.1 workaround in Python 3.x
-    if (values.dtype == np.object_ and compat.PY3):
+    if is_object_dtype(values) and compat.PY3:
 
         if values.ndim > 1:
             apply_ax = axis if axis is not None else 0
@@ -389,7 +415,7 @@ def nanmax(values, axis=None, skipna=True):
         if ((axis is not None and values.shape[axis] == 0)
                 or values.size == 0):
             try:
-                result = com.ensure_float(values.sum(axis, dtype=dtype_max))
+                result = ensure_float(values.sum(axis, dtype=dtype_max))
                 result.fill(np.nan)
             except:
                 result = np.nan
@@ -405,7 +431,7 @@ def nanargmax(values, axis=None, skipna=True):
     Returns -1 in the NA case
     """
     values, mask, dtype, _ = _get_values(values, skipna, fill_value_typ='-inf',
-                                      isfinite=True)
+                                         isfinite=True)
     result = values.argmax(axis)
     result = _maybe_arg_null_out(result, axis, mask, skipna)
     return result
@@ -416,7 +442,7 @@ def nanargmin(values, axis=None, skipna=True):
     Returns -1 in the NA case
     """
     values, mask, dtype, _ = _get_values(values, skipna, fill_value_typ='+inf',
-                                      isfinite=True)
+                                         isfinite=True)
     result = values.argmin(axis)
     result = _maybe_arg_null_out(result, axis, mask, skipna)
     return result
@@ -424,7 +450,7 @@ def nanargmin(values, axis=None, skipna=True):
 
 @disallow('M8')
 def nanskew(values, axis=None, skipna=True):
-    if not isinstance(values.dtype.type, np.floating):
+    if not _is_floating_dtype(values):
         values = values.astype('f8')
 
     mask = isnull(values)
@@ -458,7 +484,7 @@ def nanskew(values, axis=None, skipna=True):
 
 @disallow('M8')
 def nankurt(values, axis=None, skipna=True):
-    if not isinstance(values.dtype.type, np.floating):
+    if not _is_floating_dtype(values):
         values = values.astype('f8')
 
     mask = isnull(values)
@@ -493,7 +519,7 @@ def nankurt(values, axis=None, skipna=True):
 @disallow('M8')
 def nanprod(values, axis=None, skipna=True):
     mask = isnull(values)
-    if skipna and not issubclass(values.dtype.type, np.integer):
+    if skipna and not _is_any_int_dtype(values):
         values = values.copy()
         values[mask] = 1
     result = values.prod(axis)
@@ -502,7 +528,7 @@ def nanprod(values, axis=None, skipna=True):
 
 def _maybe_arg_null_out(result, axis, mask, skipna):
     # helper function for nanargmin/nanargmax
-    if axis is None:
+    if axis is None or not getattr(result, 'ndim', False):
         if skipna:
             if mask.all():
                 result = -1
@@ -520,19 +546,24 @@ def _maybe_arg_null_out(result, axis, mask, skipna):
 
 
 def _get_counts(mask, axis):
-    if axis is not None:
-        count = (mask.shape[axis] - mask.sum(axis)).astype(float)
-    else:
-        count = float(mask.size - mask.sum())
+    if axis is None:
+        return float(mask.size - mask.sum())
 
-    return count
+    count = mask.shape[axis] - mask.sum(axis)
+    try:
+        return count.astype(float)
+    except AttributeError:
+        return np.array(count, dtype=float)
 
 
 def _maybe_null_out(result, axis, mask):
-    if axis is not None:
+    if axis is not None and getattr(result, 'ndim', False):
         null_mask = (mask.shape[axis] - mask.sum(axis)) == 0
-        if null_mask.any():
-            result = result.astype('f8')
+        if np.any(null_mask):
+            if np.iscomplexobj(result):
+                result = result.astype('c16')
+            else:
+                result = result.astype('f8')
             result[null_mask] = np.nan
     else:
         null_mask = mask.size - mask.sum()
@@ -617,9 +648,17 @@ def nancov(a, b, min_periods=None):
 
 def _ensure_numeric(x):
     if isinstance(x, np.ndarray):
-        if x.dtype == np.object_:
+        if is_integer_dtype(x) or is_bool_dtype(x):
             x = x.astype(np.float64)
-    elif not (com.is_float(x) or com.is_integer(x) or com.is_complex(x)):
+        elif is_object_dtype(x):
+            try:
+                x = x.astype(np.complex128)
+            except:
+                x = x.astype(np.float64)
+            else:
+                if not np.any(x.imag):
+                    x = x.real
+    elif not (is_float(x) or is_integer(x) or is_complex(x)):
         try:
             x = float(x)
         except Exception:
@@ -643,7 +682,7 @@ def make_nancomp(op):
         result = op(x, y)
 
         if mask.any():
-            if result.dtype == np.bool_:
+            if is_bool_dtype(result):
                 result = result.astype('O')
             np.putmask(result, mask, np.nan)
 
@@ -664,16 +703,16 @@ def unique1d(values):
     """
     if np.issubdtype(values.dtype, np.floating):
         table = _hash.Float64HashTable(len(values))
-        uniques = np.array(table.unique(com._ensure_float64(values)),
+        uniques = np.array(table.unique(_ensure_float64(values)),
                            dtype=np.float64)
     elif np.issubdtype(values.dtype, np.datetime64):
         table = _hash.Int64HashTable(len(values))
-        uniques = table.unique(com._ensure_int64(values))
+        uniques = table.unique(_ensure_int64(values))
         uniques = uniques.view('M8[ns]')
     elif np.issubdtype(values.dtype, np.integer):
         table = _hash.Int64HashTable(len(values))
-        uniques = table.unique(com._ensure_int64(values))
+        uniques = table.unique(_ensure_int64(values))
     else:
         table = _hash.PyObjectHashTable(len(values))
-        uniques = table.unique(com._ensure_object(values))
+        uniques = table.unique(_ensure_object(values))
     return uniques
