@@ -417,7 +417,7 @@ class Panel(NDFrame):
             Column label for index column(s) if desired. If None is given, and
             `header` and `index` are True, then the index names are used. A
             sequence should be given if the DataFrame uses MultiIndex.
-        startow : upper left cell row to dump data frame
+        startrow : upper left cell row to dump data frame
         startcol : upper left cell column to dump data frame
 
         Notes
@@ -797,12 +797,15 @@ class Panel(NDFrame):
         axis : int
         """
 
-        key = self._get_axis(axis)[i]
+        ax = self._get_axis(axis)
+        key = ax[i]
 
         # xs cannot handle a non-scalar key, so just reindex here
-        if _is_list_like(key):
-            indexer = {self._get_axis_name(axis): key}
-            return self.reindex(**indexer)
+        # if we have a multi-index and a single tuple, then its a reduction (GH 7516)
+        if not (isinstance(ax, MultiIndex) and isinstance(key, tuple)):
+            if _is_list_like(key):
+                indexer = {self._get_axis_name(axis): key}
+                return self.reindex(**indexer)
 
         # a reduction
         if axis == 0:
