@@ -2,13 +2,17 @@
 
 from pandas.compat import u, range, map, openpyxl_compat
 from datetime import datetime, date, time
+import sys
 import os
 from distutils.version import LooseVersion
 
+import operator
+import functools
 import nose
 
 from numpy import nan
 import numpy as np
+from numpy.testing.decorators import slow
 
 from pandas import DataFrame, Index, MultiIndex
 from pandas.io.parsers import read_csv
@@ -16,6 +20,7 @@ from pandas.io.excel import (
     ExcelFile, ExcelWriter, read_excel, _XlwtWriter, _OpenpyxlWriter,
     register_writer, _XlsxWriter
 )
+from pandas.io.common import URLError
 from pandas.util.testing import ensure_clean
 from pandas.core.config import set_option, get_option
 import pandas.util.testing as tm
@@ -45,10 +50,6 @@ def _skip_if_no_openpyxl():
     except ImportError:
         raise nose.SkipTest('openpyxl not installed, skipping')
 
-    if not openpyxl_compat.is_compat():
-        raise nose.SkipTest('need %s <= openpyxl < %s, skipping' %
-                (openpyxl_compat.start_ver, openpyxl_compat.stop_ver))
-
 
 def _skip_if_no_xlsxwriter():
     try:
@@ -282,6 +283,39 @@ class ExcelReaderTests(SharedItems, tm.TestCase):
             result = read_excel(book, sheetname="SheetA", engine="xlrd")
             tm.assert_frame_equal(df, result)
 
+    @tm.network
+    def test_read_from_http_url(self):
+        _skip_if_no_xlrd()
+
+        url = ('https://raw.github.com/pydata/pandas/master/'
+               'pandas/io/tests/data/test.xlsx')
+        url_table = read_excel(url)
+        dirpath = tm.get_data_path()
+        localtable = os.path.join(dirpath, 'test.xlsx')
+        local_table = read_excel(localtable)
+        tm.assert_frame_equal(url_table, local_table)
+
+    @slow
+    def test_read_from_file_url(self):
+        _skip_if_no_xlrd()
+
+        # FILE
+        if sys.version_info[:2] < (2, 6):
+            raise nose.SkipTest("file:// not supported with Python < 2.6")
+        dirpath = tm.get_data_path()
+        localtable = os.path.join(dirpath, 'test.xlsx')
+        local_table = read_excel(localtable)
+
+        try:
+            url_table = read_excel('file://localhost/' + localtable)
+        except URLError:
+            # fails on some systems
+            raise nose.SkipTest("failing on %s" %
+                                ' '.join(platform.uname()).strip())
+
+        tm.assert_frame_equal(url_table, local_table)
+
+
     def test_xlsx_table(self):
         _skip_if_no_xlrd()
         _skip_if_no_openpyxl()
@@ -884,7 +918,6 @@ class ExcelWriterBase(SharedItems):
             result = read_excel(filename, 'TestSheet', encoding='utf8')
             tm.assert_frame_equal(result, df)
 
-
     def test_to_excel_unicode_filename(self):
         _skip_if_no_xlrd()
         with ensure_clean(u('\u0192u.') + self.ext) as filename:
@@ -1094,13 +1127,36 @@ class ExcelWriterBase(SharedItems):
             tm.assert_series_equal(write_frame['B'], read_frame['B'])
 
 
+def raise_wrapper(orig_method):
+    @functools.wraps(orig_method)
+    def wrapped(self, *args, **kwargs):
+        _skip_if_no_openpyxl()
+        if openpyxl_compat.is_compat():
+            orig_method(self, *args, **kwargs)
+        else:
+            msg = 'Installed openpyxl is not supported at this time\. Use.+'
+            with tm.assertRaisesRegexp(ValueError, msg):
+                orig_method(self, *args, **kwargs)
+    return wrapped
+
+
+def raise_on_incompat_version(cls):
+    methods = filter(operator.methodcaller('startswith', 'test_'), dir(cls))
+    for method in methods:
+        setattr(cls, method, raise_wrapper(getattr(cls, method)))
+    return cls
+
+
+@raise_on_incompat_version
 class OpenpyxlTests(ExcelWriterBase, tm.TestCase):
     ext = '.xlsx'
     engine_name = 'openpyxl'
-    check_skip = staticmethod(_skip_if_no_openpyxl)
+    check_skip = staticmethod(lambda *args, **kwargs: None)
 
     def test_to_excel_styleconverter(self):
         _skip_if_no_openpyxl()
+        if not openpyxl_compat.is_compat():
+            raise nose.SkipTest('incompatiable openpyxl version')
 
         import openpyxl
 
@@ -1114,17 +1170,17 @@ class OpenpyxlTests(ExcelWriterBase, tm.TestCase):
         xlsx_style = _OpenpyxlWriter._convert_to_style(hstyle)
         self.assertTrue(xlsx_style.font.bold)
         self.assertEqual(openpyxl.style.Border.BORDER_THIN,
-                          xlsx_style.borders.top.border_style)
+                         xlsx_style.borders.top.border_style)
         self.assertEqual(openpyxl.style.Border.BORDER_THIN,
-                          xlsx_style.borders.right.border_style)
+                         xlsx_style.borders.right.border_style)
         self.assertEqual(openpyxl.style.Border.BORDER_THIN,
-                          xlsx_style.borders.bottom.border_style)
+                         xlsx_style.borders.bottom.border_style)
         self.assertEqual(openpyxl.style.Border.BORDER_THIN,
-                          xlsx_style.borders.left.border_style)
+                         xlsx_style.borders.left.border_style)
         self.assertEqual(openpyxl.style.Alignment.HORIZONTAL_CENTER,
-                          xlsx_style.alignment.horizontal)
+                         xlsx_style.alignment.horizontal)
         self.assertEqual(openpyxl.style.Alignment.VERTICAL_TOP,
-                          xlsx_style.alignment.vertical)
+                         xlsx_style.alignment.vertical)
 
 
 class XlwtTests(ExcelWriterBase, tm.TestCase):
@@ -1160,6 +1216,7 @@ class XlsxWriterTests(ExcelWriterBase, tm.TestCase):
     check_skip = staticmethod(_skip_if_no_xlsxwriter)
 
 
+@raise_on_incompat_version
 class OpenpyxlTests_NoMerge(ExcelWriterBase, tm.TestCase):
     ext = '.xlsx'
     engine_name = 'openpyxl'
@@ -1197,6 +1254,8 @@ class ExcelWriterEngineTests(tm.TestCase):
             writer_klass = _XlsxWriter
         except ImportError:
             _skip_if_no_openpyxl()
+            if not openpyxl_compat.is_compat():
+                raise nose.SkipTest('incompatible openpyxl version')
             writer_klass = _OpenpyxlWriter
 
         with ensure_clean('.xlsx') as path:
@@ -1246,6 +1305,7 @@ class ExcelWriterEngineTests(tm.TestCase):
         check_called(lambda: df.to_excel('something.xls', engine='dummy'))
         set_option('io.excel.xlsx.writer', val)
 
+
 if __name__ == '__main__':
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                    exit=False)
