@@ -698,6 +698,9 @@ int tokenize_delimited(parser_t *self, size_t line_limit)
             } else if (c == '\r') {
                 self->state = EAT_CRNL;
                 break;
+            } else if (c == self->commentchar) {
+                self->state = EAT_LINE_COMMENT;
+                break;
             }
 
             /* normal character - handle as START_FIELD */
@@ -752,6 +755,16 @@ int tokenize_delimited(parser_t *self, size_t line_limit)
             self->state = IN_FIELD;
             break;
 
+        case EAT_LINE_COMMENT:
+            if (c == '\n') {
+                self->file_lines++;
+                self->state = START_RECORD;
+            } else if (c == '\r') {
+                self->file_lines++;
+                self->state = EAT_CRNL_NOP;
+            }
+            break;
+
         case IN_FIELD:
             /* in unquoted field */
             if (c == '\n') {
@@ -883,6 +896,15 @@ int tokenize_delimited(parser_t *self, size_t line_limit)
             }
             break;
 
+        case EAT_CRNL_NOP: /* inside an ignored comment line */
+            self->state = START_RECORD;
+            /* \r line terminator -- parse this character again */
+            if (c != '\n' && c != self->delimiter) {
+                --i;
+                --buf;
+            }
+            break;
+
         default:
             break;
 
@@ -1162,7 +1184,6 @@ int tokenize_whitespace(parser_t *self, size_t line_limit)
                 self->state = EAT_CRNL;
                 break;
             } else if (IS_WHITESPACE(c)) {
-                END_FIELD();
                 self->state = EAT_WHITESPACE;
                 break;
             } else {
@@ -1319,10 +1340,14 @@ int tokenize_whitespace(parser_t *self, size_t line_limit)
                 /* self->state = START_RECORD; */
             } else if (IS_WHITESPACE(c)){
                 // Handle \r-delimited files
-                END_LINE_AND_FIELD_STATE(EAT_WHITESPACE);
+                END_LINE_STATE(EAT_WHITESPACE);
             } else {
-                PUSH_CHAR(c);
-                END_LINE_STATE(IN_FIELD);
+                /* XXX
+                 * first character of a new record--need to back up and reread
+                 * to handle properly...
+                 */
+                i--; buf--; /* back up one character (HACK!) */
+                END_LINE_STATE(START_RECORD);
             }
             break;
 
