@@ -1,4 +1,5 @@
 
+import nose
 from numpy import nan
 import numpy as np
 from pandas import Index, isnull, Timestamp
@@ -7,7 +8,7 @@ import pandas.util.testing as tm
 from pandas.compat import range, lrange, zip
 import pandas.lib as lib
 import pandas.algos as algos
-from datetime import datetime
+
 
 class TestTseriesUtil(tm.TestCase):
     _multiprocess_can_split_ = True
@@ -335,7 +336,8 @@ def test_convert_objects_complex_number():
 
 
 def test_rank():
-    from pandas.compat.scipy import rankdata
+    tm._skip_if_no_scipy()
+    from scipy.stats import rankdata
 
     def _check(arr):
         mask = ~np.isfinite(arr)
@@ -650,6 +652,13 @@ class TestTypeInference(tm.TestCase):
         except ImportError:
             pass
 
+    def test_object(self):
+
+        # GH 7431
+        # cannot infer more than this as only a single element
+        arr = np.array([None],dtype='O')
+        result = lib.infer_dtype(arr)
+        self.assertEqual(result, 'mixed')
 
 class TestMoments(tm.TestCase):
     pass
