@@ -16,6 +16,7 @@ import pandas.lib as lib
 import pandas.core.common as com
 from pandas.core.index import Index
 
+from pandas.core.series import Series
 from pandas.tseries.index import date_range
 import pandas.tseries.tools as tools
 import pandas.tseries.frequencies as frequencies
@@ -144,7 +145,10 @@ def _dt_to_float_ordinal(dt):
     preserving hours, minutes, seconds and microseconds.  Return value
     is a :func:`float`.
     """
-    base = dates.date2num(dt)
+    if isinstance(dt, (np.ndarray, Series)) and com.is_datetime64_ns_dtype(dt):
+        base = dates.epoch2num(dt.asi8 / 1.0E9)
+    else:
+        base = dates.date2num(dt)
     return base
 
 
@@ -509,7 +513,7 @@ def _daily_finder(vmin, vmax, freq):
     def first_label(label_flags):
         if (label_flags[0] == 0) and (label_flags.size > 1) and \
                 ((vmin_orig % 1) > 0.0):
-                return label_flags[1]
+            return label_flags[1]
         else:
             return label_flags[0]
 
