@@ -13,7 +13,6 @@ import pandas.core.common as com
 import pandas.lib as lib
 import pandas.tslib as tslib
 
-
 class FreqGroup(object):
     FR_ANN = 1000
     FR_QTR = 2000
@@ -31,20 +30,40 @@ class FreqGroup(object):
 
 class Resolution(object):
 
-    RESO_US = 0
-    RESO_SEC = 1
-    RESO_MIN = 2
-    RESO_HR = 3
-    RESO_DAY = 4
+    RESO_US = tslib.US_RESO
+    RESO_MS = tslib.MS_RESO
+    RESO_SEC = tslib.S_RESO
+    RESO_MIN = tslib.T_RESO
+    RESO_HR = tslib.H_RESO
+    RESO_DAY = tslib.D_RESO
+
+    _reso_str_map = {
+    RESO_US: 'microsecond',
+    RESO_MS: 'millisecond',
+    RESO_SEC: 'second',
+    RESO_MIN: 'minute',
+    RESO_HR: 'hour',
+    RESO_DAY: 'day'}
+
+    _reso_period_map = {
+    'year': 'A',
+    'quarter': 'Q',
+    'month': 'M',
+    'day': 'D',
+    'hour': 'H',
+    'minute': 'T',
+    'second': 'S',
+    'millisecond': 'L',
+    'microsecond': 'U',
+    'nanosecond': 'N'}
 
     @classmethod
     def get_str(cls, reso):
-        return {cls.RESO_US: 'microsecond',
-                cls.RESO_SEC: 'second',
-                cls.RESO_MIN: 'minute',
-                cls.RESO_HR: 'hour',
-                cls.RESO_DAY: 'day'}.get(reso, 'day')
+        return cls._reso_str_map.get(reso, 'day')
 
+    @classmethod
+    def get_freq(cls, resostr):
+        return cls._reso_period_map[resostr]
 
 def get_reso_string(reso):
     return Resolution.get_str(reso)
@@ -240,21 +259,6 @@ for _i, _weekday in enumerate(['MON', 'TUE', 'WED', 'THU', 'FRI']):
 _legacy_reverse_map = dict((v, k) for k, v in
                            reversed(sorted(compat.iteritems(_rule_aliases))))
 
-
-def inferTimeRule(index):
-    from pandas.tseries.index import DatetimeIndex
-    import warnings
-    warnings.warn("This method is deprecated, use infer_freq or inferred_freq"
-                  " attribute of DatetimeIndex", FutureWarning)
-
-    freq = DatetimeIndex(index).inferred_freq
-    if freq is None:
-        raise Exception('Unable to infer time rule')
-
-    offset = to_offset(freq)
-    return get_legacy_offset_name(offset)
-
-
 def to_offset(freqstr):
     """
     Return DateOffset object from string representation
@@ -283,11 +287,11 @@ def to_offset(freqstr):
         try:
             for stride, name, _ in opattern.findall(freqstr):
                 offset = get_offset(name)
+                if stride_sign is None:
+                    stride_sign = -1 if stride.startswith('-') else 1
                 if not stride:
                     stride = 1
                 stride = int(stride)
-                if stride_sign is None:
-                    stride_sign = np.sign(stride)
                 offset = offset * int(np.fabs(stride) * stride_sign)
                 if delta is None:
                     delta = offset
@@ -337,7 +341,7 @@ def get_base_alias(freqstr):
     """
     return _base_and_stride(freqstr)[0]
 
-_dont_uppercase = ['MS', 'ms']
+_dont_uppercase = set(('MS', 'ms'))
 
 
 def get_offset(name):
@@ -497,7 +501,7 @@ def _period_alias_dictionary():
     H_aliases = ["H", "HR", "HOUR", "HRLY", "HOURLY"]
     T_aliases = ["T", "MIN", "MINUTE", "MINUTELY"]
     S_aliases = ["S", "SEC", "SECOND", "SECONDLY"]
-    L_aliases = ["L", "MS", "MILLISECOND", "MILLISECONDLY"]
+    L_aliases = ["L", "ms", "MILLISECOND", "MILLISECONDLY"]
     U_aliases = ["U", "US", "MICROSECOND", "MICROSECONDLY"]
     N_aliases = ["N", "NS", "NANOSECOND", "NANOSECONDLY"]
 
@@ -587,22 +591,9 @@ def _period_alias_dictionary():
 
     return alias_dict
 
-_reso_period_map = {
-    "year": "A",
-    "quarter": "Q",
-    "month": "M",
-    "day": "D",
-    "hour": "H",
-    "minute": "T",
-    "second": "S",
-    "millisecond": "L",
-    "microsecond": "U",
-    "nanosecond": "N",
-}
-
 
 def _infer_period_group(freqstr):
-    return _period_group(_reso_period_map[freqstr])
+    return _period_group(Resolution._reso_period_map[freqstr])
 
 
 def _period_group(freqstr):
@@ -615,10 +606,13 @@ _period_alias_dict = _period_alias_dictionary()
 def _period_str_to_code(freqstr):
     # hack
     freqstr = _rule_aliases.get(freqstr, freqstr)
-    freqstr = _rule_aliases.get(freqstr.lower(), freqstr)
+
+    if freqstr not in _dont_uppercase:
+        freqstr = _rule_aliases.get(freqstr.lower(), freqstr)
 
     try:
-        freqstr = freqstr.upper()
+        if freqstr not in _dont_uppercase:
+            freqstr = freqstr.upper()
         return _period_code_map[freqstr]
     except KeyError:
         try:
@@ -637,22 +631,31 @@ def infer_freq(index, warn=True):
     Parameters
     ----------
     index : DatetimeIndex
+            if passed a Series will use the values of the series (NOT THE INDEX)
     warn : boolean, default True
 
     Returns
     -------
     freq : string or None
         None if no discernible frequency
+        TypeError if the index is not datetime-like
     """
-    from pandas.tseries.index import DatetimeIndex
+    import pandas as pd
 
-    if not isinstance(index, DatetimeIndex):
-        from pandas.tseries.period import PeriodIndex
-        if isinstance(index, PeriodIndex):
-            raise ValueError("PeriodIndex given. Check the `freq` attribute "
-                             "instead of using infer_freq.")
-        index = DatetimeIndex(index)
+    if isinstance(index, com.ABCSeries):
+        values = index.values
+        if not (com.is_datetime64_dtype(index.values) or values.dtype == object):
+            raise TypeError("cannot infer freq from a non-convertible dtype on a Series of {0}".format(index.dtype))
+        index = values
+    if isinstance(index, pd.PeriodIndex):
+        raise TypeError("PeriodIndex given. Check the `freq` attribute "
+                         "instead of using infer_freq.")
+    if isinstance(index, pd.Index) and not isinstance(index, pd.DatetimeIndex):
+        if isinstance(index, (pd.Int64Index, pd.Float64Index)):
+            raise TypeError("cannot infer freq from a non-convertible index type {0}".format(type(index)))
+        index = index.values
 
+    index = pd.DatetimeIndex(index)
     inferer = _FrequencyInferer(index, warn=warn)
     return inferer.get_freq()
 
@@ -663,6 +666,25 @@ _ONE_MINUTE = 60 * _ONE_SECOND
 _ONE_HOUR = 60 * _ONE_MINUTE
 _ONE_DAY = 24 * _ONE_HOUR
 
+def _tz_convert_with_transitions(values, to_tz, from_tz):
+    """
+    convert i8 values from the specificed timezone to the to_tz zone, taking
+    into account DST transitions
+    """
+
+    # vectorization is slow, so tests if we can do this via the faster tz_convert
+    f = lambda x: tslib.tz_convert_single(x, to_tz, from_tz)
+
+    if len(values) > 2:
+        first_slow, last_slow = f(values[0]),f(values[-1])
+
+        first_fast, last_fast = tslib.tz_convert(np.array([values[0],values[-1]],dtype='i8'),to_tz,from_tz)
+
+        # don't cross a DST, so ok
+        if first_fast == first_slow and last_fast == last_slow:
+            return tslib.tz_convert(values,to_tz,from_tz)
+
+    return np.vectorize(f)(values)
 
 class _FrequencyInferer(object):
     """
@@ -672,6 +694,10 @@ class _FrequencyInferer(object):
     def __init__(self, index, warn=True):
         self.index = index
         self.values = np.asarray(index).view('i8')
+
+        if index.tz is not None:
+            self.values = _tz_convert_with_transitions(self.values,'UTC',index.tz)
+
         self.warn = warn
 
         if len(index) < 3:
