@@ -2,7 +2,7 @@ from datetime import datetime
 from pandas.compat import range
 import pickle
 import nose
-
+import sys
 import numpy as np
 
 from pandas.core.index import Index
@@ -11,23 +11,15 @@ from pandas.tseries.index import DatetimeIndex
 from pandas import Timestamp
 from pandas.tseries.offsets import generate_range
 from pandas.tseries.index import cdate_range, bdate_range, date_range
-import pandas.tseries.tools as tools
 
 import pandas.core.datetools as datetools
 from pandas.util.testing import assertRaisesRegexp
 import pandas.util.testing as tm
 
 
-def _skip_if_no_pytz():
-    try:
-        import pytz
-    except ImportError:
-        raise nose.SkipTest("pytz not installed")
-
-
-def _skip_if_no_cday():
-    if datetools.cday is None:
-        raise nose.SkipTest("CustomBusinessDay not available.")
+def _skip_if_windows_python_3():
+    if sys.version_info > (3,) and sys.platform == 'win32':
+        raise nose.SkipTest("not used on python 3/win32")
 
 
 def eq_gen_range(kwargs, expected):
@@ -45,7 +37,7 @@ class TestGenRangeGeneration(tm.TestCase):
         self.assert_numpy_array_equal(rng1, rng2)
 
     def test_generate_cday(self):
-        _skip_if_no_cday()
+        tm._skip_if_no_cday()
         rng1 = list(generate_range(START, END, offset=datetools.cday))
         rng2 = list(generate_range(START, END, time_rule='C'))
         self.assert_numpy_array_equal(rng1, rng2)
@@ -287,10 +279,15 @@ class TestDateRange(tm.TestCase):
         self.rng[2:2].summary()
 
     def test_summary_pytz(self):
-        _skip_if_no_pytz()
+        tm._skip_if_no_pytz()
         import pytz
         bdate_range('1/1/2005', '1/1/2009', tz=pytz.utc).summary()
 
+    def test_summary_dateutil(self):
+        tm._skip_if_no_dateutil()
+        import dateutil
+        bdate_range('1/1/2005', '1/1/2009', tz=dateutil.tz.tzutc()).summary()
+
     def test_misc(self):
         end = datetime(2009, 5, 13)
         dr = bdate_range(end=end, periods=20)
@@ -354,9 +351,9 @@ class TestDateRange(tm.TestCase):
         exp_values = [start + i * offset for i in range(5)]
         self.assert_numpy_array_equal(result, DatetimeIndex(exp_values))
 
-    def test_range_tz(self):
+    def test_range_tz_pytz(self):
         # GH 2906
-        _skip_if_no_pytz()
+        tm._skip_if_no_pytz()
         from pytz import timezone as tz
 
         start = datetime(2011, 1, 1, tzinfo=tz('US/Eastern'))
@@ -377,8 +374,33 @@ class TestDateRange(tm.TestCase):
         self.assertEqual(dr[0], start)
         self.assertEqual(dr[2], end)
 
-    def test_month_range_union_tz(self):
-        _skip_if_no_pytz()
+    def test_range_tz_dateutil(self):
+        # GH 2906
+        tm._skip_if_no_dateutil()
+        # Use maybe_get_tz to fix filename in tz under dateutil.
+        from pandas.tslib import maybe_get_tz
+        tz = lambda x: maybe_get_tz('dateutil/' + x)
+
+        start = datetime(2011, 1, 1, tzinfo=tz('US/Eastern'))
+        end = datetime(2011, 1, 3, tzinfo=tz('US/Eastern'))
+
+        dr = date_range(start=start, periods=3)
+        self.assert_(dr.tz == tz('US/Eastern'))
+        self.assert_(dr[0] == start)
+        self.assert_(dr[2] == end)
+
+        dr = date_range(end=end, periods=3)
+        self.assert_(dr.tz == tz('US/Eastern'))
+        self.assert_(dr[0] == start)
+        self.assert_(dr[2] == end)
+
+        dr = date_range(start=start, end=end)
+        self.assert_(dr.tz == tz('US/Eastern'))
+        self.assert_(dr[0] == start)
+        self.assert_(dr[2] == end)
+
+    def test_month_range_union_tz_pytz(self):
+        tm._skip_if_no_pytz()
         from pytz import timezone
         tz = timezone('US/Eastern')
 
@@ -393,6 +415,23 @@ class TestDateRange(tm.TestCase):
 
         early_dr.union(late_dr)
 
+    def test_month_range_union_tz_dateutil(self):
+        _skip_if_windows_python_3()
+        tm._skip_if_no_dateutil()
+        from dateutil.tz import gettz as timezone
+        tz = timezone('US/Eastern')
+
+        early_start = datetime(2011, 1, 1)
+        early_end = datetime(2011, 3, 1)
+
+        late_start = datetime(2011, 3, 1)
+        late_end = datetime(2011, 5, 1)
+
+        early_dr = date_range(start=early_start, end=early_end, tz=tz, freq=datetools.monthEnd)
+        late_dr = date_range(start=late_start, end=late_end, tz=tz, freq=datetools.monthEnd)
+
+        early_dr.union(late_dr)
+
     def test_range_closed(self):
         begin = datetime(2011, 1, 1)
         end = datetime(2014, 1, 1)
@@ -412,7 +451,7 @@ class TestDateRange(tm.TestCase):
 class TestCustomDateRange(tm.TestCase):
 
     def setUp(self):
-        _skip_if_no_cday()
+        tm._skip_if_no_cday()
         self.rng = cdate_range(START, END)
 
     def test_constructor(self):
@@ -576,10 +615,15 @@ class TestCustomDateRange(tm.TestCase):
         self.rng[2:2].summary()
 
     def test_summary_pytz(self):
-        _skip_if_no_pytz()
+        tm._skip_if_no_pytz()
         import pytz
         cdate_range('1/1/2005', '1/1/2009', tz=pytz.utc).summary()
 
+    def test_summary_dateutil(self):
+        tm._skip_if_no_dateutil()
+        import dateutil
+        cdate_range('1/1/2005', '1/1/2009', tz=dateutil.tz.tzutc()).summary()
+
     def test_misc(self):
         end = datetime(2009, 5, 13)
         dr = cdate_range(end=end, periods=20)
