@@ -7,15 +7,14 @@ import nose
 
 import numpy as np
 
-from pandas import Index, DatetimeIndex, Timestamp, date_range, period_range
+from pandas import Index, DatetimeIndex, Timestamp, Series, date_range, period_range
 
 from pandas.tseries.frequencies import to_offset, infer_freq
 from pandas.tseries.tools import to_datetime
 import pandas.tseries.frequencies as fmod
 import pandas.tseries.offsets as offsets
 from pandas.tseries.period import PeriodIndex
-
-import pandas.lib as lib
+import pandas.compat as compat
 
 from pandas import _np_version_under1p7
 import pandas.util.testing as tm
@@ -74,6 +73,16 @@ def test_to_offset_negative():
     assert(result.n == -310)
 
 
+def test_to_offset_leading_zero():
+    freqstr = '00H 00T 01S'
+    result = to_offset(freqstr)
+    assert(result.n == 1)
+
+    freqstr = '-00H 03T 14S'
+    result = to_offset(freqstr)
+    assert(result.n == -194)
+
+
 def test_anchored_shortcuts():
     result = to_offset('W')
     expected = to_offset('W-SUN')
@@ -91,7 +100,7 @@ class TestFrequencyInference(tm.TestCase):
 
     def test_raise_if_period_index(self):
         index = PeriodIndex(start="1/1/1990", periods=20, freq="M")
-        self.assertRaises(ValueError, infer_freq, index)
+        self.assertRaises(TypeError, infer_freq, index)
 
     def test_raise_if_too_few(self):
         index = _dti(['12/31/1998', '1/3/1999'])
@@ -99,18 +108,18 @@ class TestFrequencyInference(tm.TestCase):
 
     def test_business_daily(self):
         index = _dti(['12/31/1998', '1/3/1999', '1/4/1999'])
-        self.assert_(infer_freq(index) == 'B')
+        self.assertEqual(infer_freq(index), 'B')
 
     def test_day(self):
         self._check_tick(timedelta(1), 'D')
 
     def test_day_corner(self):
         index = _dti(['1/1/2000', '1/2/2000', '1/3/2000'])
-        self.assert_(infer_freq(index) == 'D')
+        self.assertEqual(infer_freq(index), 'D')
 
     def test_non_datetimeindex(self):
         dates = to_datetime(['1/1/2000', '1/2/2000', '1/3/2000'])
-        self.assert_(infer_freq(dates) == 'D')
+        self.assertEqual(infer_freq(dates), 'D')
 
     def test_hour(self):
         self._check_tick(timedelta(hours=1), 'H')
@@ -128,8 +137,7 @@ class TestFrequencyInference(tm.TestCase):
         self._check_tick(timedelta(microseconds=1), 'U')
 
     def test_nanosecond(self):
-        if _np_version_under1p7:
-            raise nose.SkipTest("requires numpy >= 1.7 to run")
+        tm._skip_if_not_numpy17_friendly()
         self._check_tick(np.timedelta64(1, 'ns'), 'N')
 
     def _check_tick(self, base_delta, code):
@@ -141,15 +149,15 @@ class TestFrequencyInference(tm.TestCase):
                 exp_freq = '%d%s' % (i, code)
             else:
                 exp_freq = code
-            self.assert_(infer_freq(index) == exp_freq)
+            self.assertEqual(infer_freq(index), exp_freq)
 
         index = _dti([b + base_delta * 7] +
                      [b + base_delta * j for j in range(3)])
-        self.assert_(infer_freq(index) is None)
+        self.assertIsNone(infer_freq(index))
 
         index = _dti([b + base_delta * j for j in range(3)] +
                      [b + base_delta * 7])
-        self.assert_(infer_freq(index) is None)
+        self.assertIsNone(infer_freq(index))
 
     def test_weekly(self):
         days = ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN']
@@ -174,7 +182,7 @@ class TestFrequencyInference(tm.TestCase):
 
     def test_monthly_ambiguous(self):
         rng = _dti(['1/31/2000', '2/29/2000', '3/31/2000'])
-        self.assert_(rng.inferred_freq == 'M')
+        self.assertEqual(rng.inferred_freq, 'M')
 
     def test_business_monthly(self):
         self._check_generated_range('1/1/2000', 'BM')
@@ -196,7 +204,7 @@ class TestFrequencyInference(tm.TestCase):
 
     def test_annual_ambiguous(self):
         rng = _dti(['1/31/2000', '1/31/2001', '1/31/2002'])
-        self.assert_(rng.inferred_freq == 'A-JAN')
+        self.assertEqual(rng.inferred_freq, 'A-JAN')
 
     def _check_generated_range(self, start, freq):
         freq = freq.upper()
@@ -207,49 +215,66 @@ class TestFrequencyInference(tm.TestCase):
             self.assertEqual(infer_freq(index), gen.freqstr)
         else:
             inf_freq = infer_freq(index)
-            self.assert_((inf_freq == 'Q-DEC' and
-                          gen.freqstr in ('Q', 'Q-DEC', 'Q-SEP', 'Q-JUN',
-                                          'Q-MAR'))
-                         or
-                         (inf_freq == 'Q-NOV' and
-                          gen.freqstr in ('Q-NOV', 'Q-AUG', 'Q-MAY', 'Q-FEB'))
-                         or
-                         (inf_freq == 'Q-OCT' and
-                          gen.freqstr in ('Q-OCT', 'Q-JUL', 'Q-APR', 'Q-JAN')))
+            self.assertTrue((inf_freq == 'Q-DEC' and
+                             gen.freqstr in ('Q', 'Q-DEC', 'Q-SEP', 'Q-JUN',
+                                             'Q-MAR'))
+                            or
+                            (inf_freq == 'Q-NOV' and
+                             gen.freqstr in ('Q-NOV', 'Q-AUG', 'Q-MAY', 'Q-FEB'))
+                            or
+                            (inf_freq == 'Q-OCT' and
+                             gen.freqstr in ('Q-OCT', 'Q-JUL', 'Q-APR', 'Q-JAN')))
 
         gen = date_range(start, periods=5, freq=freq)
         index = _dti(gen.values)
         if not freq.startswith('Q-'):
-            self.assert_(infer_freq(index) == gen.freqstr)
+            self.assertEqual(infer_freq(index), gen.freqstr)
         else:
             inf_freq = infer_freq(index)
-            self.assert_((inf_freq == 'Q-DEC' and
-                          gen.freqstr in ('Q', 'Q-DEC', 'Q-SEP', 'Q-JUN',
-                                          'Q-MAR'))
-                         or
-                         (inf_freq == 'Q-NOV' and
-                          gen.freqstr in ('Q-NOV', 'Q-AUG', 'Q-MAY', 'Q-FEB'))
-                         or
-                         (inf_freq == 'Q-OCT' and
-                          gen.freqstr in ('Q-OCT', 'Q-JUL', 'Q-APR', 'Q-JAN')))
+            self.assertTrue((inf_freq == 'Q-DEC' and
+                             gen.freqstr in ('Q', 'Q-DEC', 'Q-SEP', 'Q-JUN',
+                                             'Q-MAR'))
+                            or
+                            (inf_freq == 'Q-NOV' and
+                             gen.freqstr in ('Q-NOV', 'Q-AUG', 'Q-MAY', 'Q-FEB'))
+                            or
+                            (inf_freq == 'Q-OCT' and
+                             gen.freqstr in ('Q-OCT', 'Q-JUL', 'Q-APR', 'Q-JAN')))
 
     def test_infer_freq(self):
         rng = period_range('1959Q2', '2009Q3', freq='Q')
         rng = Index(rng.to_timestamp('D', how='e').asobject)
-        self.assert_(rng.inferred_freq == 'Q-DEC')
+        self.assertEqual(rng.inferred_freq, 'Q-DEC')
 
         rng = period_range('1959Q2', '2009Q3', freq='Q-NOV')
         rng = Index(rng.to_timestamp('D', how='e').asobject)
-        self.assert_(rng.inferred_freq == 'Q-NOV')
+        self.assertEqual(rng.inferred_freq, 'Q-NOV')
 
         rng = period_range('1959Q2', '2009Q3', freq='Q-OCT')
         rng = Index(rng.to_timestamp('D', how='e').asobject)
-        self.assert_(rng.inferred_freq == 'Q-OCT')
+        self.assertEqual(rng.inferred_freq, 'Q-OCT')
+
+    def test_infer_freq_tz(self):
+
+        freqs = {'AS-JAN': ['2009-01-01', '2010-01-01', '2011-01-01', '2012-01-01'],
+                 'Q-OCT': ['2009-01-31', '2009-04-30', '2009-07-31', '2009-10-31'],
+                 'M': ['2010-11-30', '2010-12-31', '2011-01-31', '2011-02-28'],
+                 'W-SAT': ['2010-12-25', '2011-01-01', '2011-01-08', '2011-01-15'],
+                 'D': ['2011-01-01', '2011-01-02', '2011-01-03', '2011-01-04'],
+                 'H': ['2011-12-31 22:00', '2011-12-31 23:00', '2012-01-01 00:00', '2012-01-01 01:00']
+        }
+
+        # GH 7310
+        for tz in [None, 'Australia/Sydney', 'Asia/Tokyo', 'Europe/Paris',
+                   'US/Pacific', 'US/Eastern']:
+            for expected, dates in compat.iteritems(freqs):
+                idx = DatetimeIndex(dates, tz=tz)
+                self.assertEqual(idx.inferred_freq, expected)
 
     def test_not_monotonic(self):
         rng = _dti(['1/31/2000', '1/31/2001', '1/31/2002'])
         rng = rng[::-1]
-        self.assert_(rng.inferred_freq is None)
+        self.assertIsNone(rng.inferred_freq)
 
     def test_non_datetimeindex(self):
         rng = _dti(['1/31/2000', '1/31/2001', '1/31/2002'])
@@ -259,6 +284,53 @@ class TestFrequencyInference(tm.TestCase):
         result = infer_freq(vals)
         self.assertEqual(result, rng.inferred_freq)
 
+    def test_invalid_index_types(self):
+
+        # test all index types
+        for i in [ tm.makeIntIndex(10),
+                   tm.makeFloatIndex(10),
+                   tm.makePeriodIndex(10) ]:
+            self.assertRaises(TypeError, lambda : infer_freq(i))
+
+        for i in [ tm.makeStringIndex(10),
+                   tm.makeUnicodeIndex(10) ]:
+            self.assertRaises(ValueError, lambda : infer_freq(i))
+
+    def test_string_datetimelike_compat(self):
+
+        # GH 6463
+        expected = infer_freq(['2004-01', '2004-02', '2004-03', '2004-04'])
+        result = infer_freq(Index(['2004-01', '2004-02', '2004-03', '2004-04']))
+        self.assertEqual(result,expected)
+
+    def test_series(self):
+
+        # GH6407
+        # inferring series
+
+        # invalid type of Series
+        for s in [ Series(np.arange(10)),
+                   Series(np.arange(10.))]:
+            self.assertRaises(TypeError, lambda : infer_freq(s))
+
+        # a non-convertible string
+        self.assertRaises(ValueError, lambda : infer_freq(Series(['foo','bar'])))
+
+        # cannot infer on PeriodIndex
+        for freq in [None, 'L', 'Y']:
+            s = Series(period_range('2013',periods=10,freq=freq))
+            self.assertRaises(TypeError, lambda : infer_freq(s))
+
+        # DateTimeIndex
+        for freq in ['M', 'L', 'S']:
+            s = Series(date_range('20130101',periods=10,freq=freq))
+            inferred = infer_freq(s)
+            self.assertEqual(inferred,freq)
+
+        s = Series(date_range('20130101','20130110'))
+        inferred = infer_freq(s)
+        self.assertEqual(inferred,'D')
+
 MONTHS = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP',
           'OCT', 'NOV', 'DEC']
 
