@@ -57,9 +57,7 @@ def _infer_tzinfo(start, end):
 
 
 def _maybe_get_tz(tz, date=None):
-    if isinstance(tz, compat.string_types):
-        import pytz
-        tz = pytz.timezone(tz)
+    tz = tslib.maybe_get_tz(tz)
     if com.is_integer(tz):
         import pytz
         tz = pytz.FixedOffset(tz / 60)
@@ -71,6 +69,7 @@ def _maybe_get_tz(tz, date=None):
 
     return tz
 
+
 def _guess_datetime_format(dt_str, dayfirst=False,
                            dt_str_parse=compat.parse_date,
                            dt_str_split=_DATEUTIL_LEXER_SPLIT):
@@ -509,8 +508,11 @@ def dateutil_parse(timestr, default,
     if reso is None:
         raise ValueError("Cannot parse date.")
 
-    if reso == 'microsecond' and repl['microsecond'] == 0:
-        reso = 'second'
+    if reso == 'microsecond':
+        if repl['microsecond'] == 0:
+            reso = 'second'
+        elif repl['microsecond'] % 1000 == 0:
+            reso = 'millisecond'
 
     ret = default.replace(**repl)
     if res.weekday is not None and not res.day:
