@@ -7,7 +7,7 @@ common_setup = """from pandas_vb_common import *
 """
 
 #----------------------------------------------------------------------
-# Series.__getitem__, get_value
+# Series.__getitem__, get_value, __getitem__(slice)
 
 setup = common_setup + """
 tm.N = 1000
@@ -29,6 +29,24 @@ bm_df_getitem3 = Benchmark(statement, setup,
                            name='series_get_value',
                            start_date=datetime(2011, 11, 12))
 
+
+setup = common_setup + """
+index = tm.makeStringIndex(1000000)
+s = Series(np.random.rand(1000000), index=index)
+"""
+series_getitem_pos_slice = Benchmark("s[:800000]", setup,
+                                     name="series_getitem_pos_slice")
+
+
+setup = common_setup + """
+index = tm.makeStringIndex(1000000)
+s = Series(np.random.rand(1000000), index=index)
+lbl = s.index[800000]
+"""
+series_getitem_label_slice = Benchmark("s[:lbl]", setup,
+                                       name="series_getitem_label_slice")
+
+
 #----------------------------------------------------------------------
 # DataFrame __getitem__
 
@@ -82,6 +100,25 @@ indexing_frame_get_value = Benchmark("df.get_value(idx,col)", setup,
                                      name='indexing_frame_get_value',
                                      start_date=datetime(2011, 11, 12))
 
+setup = common_setup + """
+mi = MultiIndex.from_tuples([(x,y) for x in range(1000) for y in range(1000)])
+s =  Series(np.random.randn(1000000), index=mi)
+"""
+
+series_xs_mi_ix = Benchmark("s.ix[999]", setup,
+                            name='series_xs_mi_ix',
+                            start_date=datetime(2013, 1, 1))
+
+setup = common_setup + """
+mi = MultiIndex.from_tuples([(x,y) for x in range(1000) for y in range(1000)])
+s =  Series(np.random.randn(1000000), index=mi)
+df = DataFrame(s)
+"""
+
+frame_xs_mi_ix = Benchmark("df.ix[999]", setup,
+                           name='frame_xs_mi_ix',
+                           start_date=datetime(2013, 1, 1))
+
 #----------------------------------------------------------------------
 # Boolean DataFrame row selection
 
