@@ -12,12 +12,14 @@ modules = ['attrs_caching',
            'index_object',
            'indexing',
            'io_bench',
+           'inference',
            'hdfstore_bench',
            'join_merge',
            'miscellaneous',
            'panel_ctor',
            'packers',
            'parser_vb',
+           'panel_methods',
            'plotting',
            'reindex',
            'replace',
@@ -26,6 +28,7 @@ modules = ['attrs_caching',
            'reshape',
            'stat_ops',
            'timeseries',
+           'timedelta',
            'eval']
 
 by_module = {}
